import type { ModuleInstance } from '../module/baseModule';
import type { LegendModule, RootModule } from '../module/coreModules';
import type { Module } from '../module/module';
import type { ModuleContext } from '../module/moduleContext';
import type { AgChartInstance, AgChartOptions } from '../options/agChartOptions';
import { BBox } from '../scene/bbox';
import { Group } from '../scene/group';
import { Scene } from '../scene/scene';
import { CallbackCache } from '../util/callbackCache';
import type { PlacedLabel } from '../util/labelPlacement';
import { Observable } from '../util/observable';
import { Padding } from '../util/padding';
import type { Caption } from './caption';
import type { ChartAxis } from './chartAxis';
import { ChartHighlight } from './chartHighlight';
import type { ChartMode } from './chartMode';
import { ChartUpdateType } from './chartUpdateType';
import { DataService } from './dataService';
import { AnimationManager } from './interaction/animationManager';
import { ChartEventManager } from './interaction/chartEventManager';
import { CursorManager } from './interaction/cursorManager';
import type { HighlightChangeEvent } from './interaction/highlightManager';
import { HighlightManager } from './interaction/highlightManager';
import type { InteractionEvent } from './interaction/interactionManager';
import { InteractionManager } from './interaction/interactionManager';
import { TooltipManager } from './interaction/tooltipManager';
import { ZoomManager } from './interaction/zoomManager';
import { LayoutService } from './layout/layoutService';
import type { CategoryLegendDatum, ChartLegend } from './legendDatum';
import type { SeriesOptionsTypes } from './mapping/types';
import { ChartOverlays } from './overlay/chartOverlays';
import type { Series } from './series/series';
import { SeriesLayerManager } from './series/seriesLayerManager';
import { SeriesStateManager } from './series/seriesStateManager';
import type { ISeries, SeriesNodeDatum } from './series/seriesTypes';
import { Tooltip } from './tooltip/tooltip';
import { UpdateService } from './updateService';
type OptionalHTMLElement = HTMLElement | undefined | null;
export type TransferableResources = {
    container?: OptionalHTMLElement;
    scene: Scene;
    element: HTMLElement;
};
export interface ChartSpecialOverrides {
    document?: Document;
    window?: Window;
    overrideDevicePixelRatio?: number;
    sceneMode?: 'simple';
}
export type ChartExtendedOptions = AgChartOptions & ChartSpecialOverrides;
declare class SeriesArea {
    clip?: boolean;
    padding: Padding;
}
export declare abstract class Chart extends Observable implements AgChartInstance {
    readonly id: string;
    className?: string;
    processedOptions: AgChartOptions & {
        type?: SeriesOptionsTypes['type'];
    };
    userOptions: AgChartOptions;
    queuedUserOptions: AgChartOptions[];
    getOptions(): AgChartOptions;
    readonly scene: Scene;
    readonly seriesRoot: Group;
    legend: ChartLegend | undefined;
    readonly tooltip: Tooltip;
    readonly overlays: ChartOverlays;
    readonly highlight: ChartHighlight;
    private readonly debug;
    private extraDebugStats;
    container: OptionalHTMLElement;
    data: any;
    width?: number;
    height?: number;
    autoSize: boolean;
    private _lastAutoSize?;
    private _firstAutoSize;
    private autoSizeChanged;
    download(fileName?: string, fileFormat?: string): void;
    padding: Padding;
    seriesArea: SeriesArea;
    title?: Caption;
    subtitle?: Caption;
    footnote?: Caption;
    mode: ChartMode;
    private _destroyed;
    private readonly _destroyFns;
    get destroyed(): boolean;
    protected readonly animationManager: AnimationManager;
    protected readonly chartEventManager: ChartEventManager;
    protected readonly cursorManager: CursorManager;
    protected readonly highlightManager: HighlightManager;
    protected readonly interactionManager: InteractionManager;
    protected readonly tooltipManager: TooltipManager;
    protected readonly zoomManager: ZoomManager;
    protected readonly layoutService: LayoutService;
    protected readonly updateService: UpdateService;
    protected readonly dataService: DataService<Series<any>>;
    protected readonly axisGridGroup: Group;
    protected readonly axisGroup: Group;
    protected readonly callbackCache: CallbackCache;
    protected readonly seriesStateManager: SeriesStateManager;
    protected readonly seriesLayerManager: SeriesLayerManager;
    protected readonly modules: Record<string, {
        instance: ModuleInstance;
    }>;
    protected readonly legendModules: Record<string, {
        instance: ModuleInstance;
    }>;
    private readonly specialOverrides;
    private readonly processors;
    protected constructor(specialOverrides: ChartExtendedOptions, resources?: TransferableResources);
    addModule(module: RootModule): void;
    removeModule(module: {
        optionsKey: string;
    }): void;
    private legends;
    private attachLegend;
    addLegendModule(module: LegendModule): void;
    removeLegendModule(module: LegendModule): void;
    isModuleEnabled(module: Module): boolean;
    getModuleContext(): ModuleContext;
    destroy(opts?: {
        keepTransferableResources: boolean;
    }): TransferableResources | undefined;
    disablePointer(highlightOnly?: boolean): void;
    requestFactoryUpdate(cb: () => Promise<void>): void;
    private _pendingFactoryUpdatesCount;
    private _performUpdateNoRenderCount;
    private _performUpdateType;
    private _performUpdateSkipAnimations?;
    get performUpdateType(): ChartUpdateType;
    private _lastPerformUpdateError?;
    get lastPerformUpdateError(): Error | undefined;
    private updateShortcutCount;
    private seriesToUpdate;
    private updateMutex;
    private updateRequestors;
    private performUpdateTrigger;
    update(type?: ChartUpdateType, opts?: {
        forceNodeDataRefresh?: boolean;
        skipAnimations?: boolean;
        newAnimationBatch?: boolean;
        seriesToUpdate?: Iterable<ISeries<any>>;
        backOffMs?: number;
    }): void;
    private performUpdate;
    private checkUpdateShortcut;
    private checkFirstAutoSize;
    readonly element: HTMLElement;
    protected _axes: ChartAxis[];
    set axes(values: ChartAxis[]);
    get axes(): ChartAxis[];
    protected _series: Series<any>[];
    set series(values: Series<any>[]);
    get series(): Series<any>[];
    private addSeries;
    private initSeries;
    private removeAllSeries;
    private addSeriesListeners;
    updateAllSeriesListeners(): void;
    protected assignSeriesToAxes(): void;
    protected assignAxesToSeries(): void;
    private findMatchingAxis;
    private rawResize;
    private resize;
    processData(): Promise<void>;
    placeLabels(): Map<Series<any>, PlacedLabel[]>;
    private updateLegend;
    protected validateCategoryLegendData(legendData: CategoryLegendDatum[]): void;
    private processLayout;
    protected performLayout(): Promise<BBox>;
    protected hoverRect?: BBox;
    protected seriesRect?: BBox;
    protected animationRect?: BBox;
    private pickSeriesNode;
    lastPick?: {
        datum: SeriesNodeDatum;
        event?: Event;
    };
    protected onMouseMove(event: InteractionEvent<'hover'>): void;
    protected onLeave(event: InteractionEvent<'leave'>): void;
    private lastInteractionEvent?;
    private pointerScheduler;
    protected handlePointer(event: InteractionEvent<'hover'>): void;
    protected handlePointerTooltip(event: InteractionEvent<'hover'>, disablePointer: (highlightOnly?: boolean) => void): void;
    protected handlePointerNode(event: InteractionEvent<'hover'>): void;
    protected onClick(event: InteractionEvent<'click'>): void;
    protected onDoubleClick(event: InteractionEvent<'dblclick'>): void;
    private checkSeriesNodeClick;
    private checkSeriesNodeDoubleClick;
    private checkSeriesNodeRange;
    private onSeriesNodeClick;
    private onSeriesNodeDoubleClick;
    changeHighlightDatum(event: HighlightChangeEvent): void;
    waitForUpdate(timeoutMs?: number): Promise<void>;
    protected handleNoDataOverlay(): void;
    protected getMinRect(): BBox | undefined;
}
export {};
